@@ -1,5 +1,5 @@
 /* crl.c - CRL parser
- *      Copyright (C) 2002, 2004 g10 Code GmbH
+ *      Copyright (C) 2002, 2004, 2005 g10 Code GmbH
  *
  * This file is part of KSBA.
  *
@@ -15,7 +15,8 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301,
+ * USA.
  */
 
 #include <config.h>
@@ -34,7 +35,12 @@
 #include "ber-decoder.h"
 #include "crl.h"
 
+#if !HAVE_DECL_GPG_ERR_UNKNOWN_CRIT_EXTN
+#define GPG_ERR_UNKNOWN_CRIT_EXTN  172
+#endif
+
 static const char oidstr_crlNumber[] = "2.5.29.20";
+static const char oidstr_crlReason[] = "2.5.29.21";
 static const char oidstr_authorityKeyIdentifier[] = "2.5.29.35";
 
 /* We better buffer the hashing. */
@@ -339,9 +345,9 @@ ksba_crl_get_extension (ksba_crl_t crl, int idx,
 
 
 /* Return the authorityKeyIdentifier in r_name and r_serial or in
-   r_keyID.  Note that r_keyID is not yet supported and must be passed
-   as NULL.  GPG_ERR_NO_DATA is returned if no authorityKeyIdentifier
-   or only one using the keyIdentifier method is available. 
+   r_keyID.  GPG_ERR_NO_DATA is returned if no authorityKeyIdentifier
+   or only one using the keyIdentifier method is available and R_KEYID
+   is NULL.
 
    FIXME: This function shares a lot of code with the one in cert.c 
 */
@@ -354,13 +360,15 @@ ksba_crl_get_auth_key_id (ksba_crl_t crl,
   gpg_error_t err;
   size_t derlen;
   const unsigned char *der;
+  const unsigned char *keyid_der = NULL;
+  size_t keyid_derlen = 0;
   struct tag_info ti;
   char numbuf[30];
   size_t numbuflen;
   crl_extn_t e;
 
   if (r_keyid)
-    return gpg_error (GPG_ERR_NOT_IMPLEMENTED);
+    *r_keyid = NULL;
   if (!crl || !r_name || !r_serial)
     return gpg_error (GPG_ERR_INV_VALUE);
   *r_name = NULL;
@@ -399,17 +407,23 @@ ksba_crl_get_auth_key_id (ksba_crl_t crl,
   if (err)
     return err;
   if (ti.class != CLASS_CONTEXT) 
-    return gpg_error (GPG_ERR_INV_CRL_OBJ); /* we expected a tag */
+    return gpg_error (GPG_ERR_INV_CRL_OBJ); /* We expected a tag. */
   if (ti.ndef)
     return gpg_error (GPG_ERR_NOT_DER_ENCODED);
   if (derlen < ti.length)
     return gpg_error (GPG_ERR_BAD_BER);
 
   if (ti.tag == 0)
-    { /* We do not support the keyIdentifier method yet, but we need
-         to skip it. */
+    { /* keyIdentifier:  Just save it away for later use. */
+      keyid_der = der;
+      keyid_derlen = ti.length;
+
       der += ti.length;
       derlen -= ti.length;
+      /* If the keyid has been requested but no other data follows, we
+         directly jump to the end. */
+      if (r_keyid && !derlen)
+        goto build_keyid;
       if (!derlen)
         return gpg_error (GPG_ERR_NO_DATA); /* not available */
         
@@ -458,6 +472,19 @@ ksba_crl_get_auth_key_id (ksba_crl_t crl,
   (*r_serial)[numbuflen + ti.length] = ')';
   (*r_serial)[numbuflen + ti.length + 1] = 0;
 
+ build_keyid:
+  if (r_keyid && keyid_der && keyid_derlen)
+    {
+      sprintf (numbuf,"(%u:", (unsigned int)keyid_derlen);
+      numbuflen = strlen (numbuf);
+      *r_keyid = xtrymalloc (numbuflen + keyid_derlen + 2);
+      if (!*r_keyid)
+        return gpg_error (GPG_ERR_ENOMEM);
+      strcpy (*r_keyid, numbuf);
+      memcpy (*r_keyid+numbuflen, keyid_der, keyid_derlen);
+      (*r_keyid)[numbuflen + keyid_derlen] = ')';
+      (*r_keyid)[numbuflen + keyid_derlen + 1] = 0;
+    }
   return 0;
 }
 
@@ -971,9 +998,10 @@ parse_to_next_update (ksba_crl_t crl)
   if (err)
     return err;
   HASH (tmpbuf, ti.nhdr+ti.length);
-  _ksba_asntime_to_iso (tmpbuf+ti.nhdr, ti.length, crl->this_update);
+  _ksba_asntime_to_iso (tmpbuf+ti.nhdr, ti.length,
+                        ti.tag == TYPE_UTC_TIME, crl->this_update);
 
-  /* read the optional nextUpdate time */
+  /* Read the optional nextUpdate time. */
   err = _ksba_ber_read_tl (crl->reader, &ti);
   if (err)
     return err;
@@ -997,13 +1025,14 @@ parse_to_next_update (ksba_crl_t crl)
       if (err)
         return err;
       HASH (tmpbuf, ti.nhdr+ti.length);
-      _ksba_asntime_to_iso (tmpbuf+ti.nhdr, ti.length, crl->next_update);
+      _ksba_asntime_to_iso (tmpbuf+ti.nhdr, ti.length,
+                            ti.tag == TYPE_UTC_TIME, crl->next_update);
       err = _ksba_ber_read_tl (crl->reader, &ti);
       if (err)
         return err;
     }
 
-  /* read the first sequence tag of the optional SEQ of SEQ */
+  /* Read the first sequence tag of the optional SEQ of SEQ. */
   if (tbs_ndef || tbs_len)
     {
       if (ti.class == CLASS_UNIVERSAL && ti.tag == TYPE_SEQUENCE
@@ -1029,7 +1058,7 @@ parse_to_next_update (ksba_crl_t crl)
         }
     }
 
-  /* we need to save some stuff for the next round */
+  /* We need to save some stuff for the next round. */
   crl->state.ti = ti;
   crl->state.outer_ndef = outer_ndef;
   crl->state.outer_len = outer_len;
@@ -1040,6 +1069,78 @@ parse_to_next_update (ksba_crl_t crl)
 }
 
 
+/* Parse an enumerated value.  Note that this code is duplication of
+   the one at ocsp.c.  */
+static gpg_error_t
+parse_enumerated (unsigned char const **buf, size_t *len, struct tag_info *ti,
+                  size_t maxlen)
+{
+  gpg_error_t err;
+
+  err = _ksba_ber_parse_tl (buf, len, ti);
+  if (err)
+     ;
+  else if (!(ti->class == CLASS_UNIVERSAL && ti->tag == TYPE_ENUMERATED
+             && !ti->is_constructed) )
+    err = gpg_error (GPG_ERR_INV_OBJ);
+  else if (!ti->length)
+    err = gpg_error (GPG_ERR_TOO_SHORT);
+  else if (maxlen && ti->length > maxlen)
+    err = gpg_error (GPG_ERR_TOO_LARGE);
+  else if (ti->length > *len)
+    err = gpg_error (GPG_ERR_BAD_BER);
+
+  return err;
+}
+
+
+
+/* Store an entry extension into the current item. */
+static gpg_error_t
+store_one_entry_extension (ksba_crl_t crl,
+                           const unsigned char *der, size_t derlen)
+{
+  gpg_error_t err;
+  char *oid;
+  int critical;
+  size_t off, len;
+
+  err = parse_one_extension (der, derlen, &oid, &critical, &off, &len);
+  if (err)
+    return err;
+  if (!strcmp (oid, oidstr_crlReason))
+    {
+      struct tag_info ti;
+      const unsigned char *buf = der+off;
+      size_t mylen = len;
+
+      err = parse_enumerated (&buf, &mylen, &ti, 1);
+      if (err)
+        return err;
+      /* Note that we OR the values so that in case this extension is
+         repeated we can track all reason codes. */ 
+      switch (*buf)
+        { 
+        case  0: crl->item.reason |= KSBA_CRLREASON_UNSPECIFIED; break;
+        case  1: crl->item.reason |= KSBA_CRLREASON_KEY_COMPROMISE; break;
+        case  2: crl->item.reason |= KSBA_CRLREASON_CA_COMPROMISE; break;
+        case  3: crl->item.reason |= KSBA_CRLREASON_AFFILIATION_CHANGED; break;
+        case  4: crl->item.reason |= KSBA_CRLREASON_SUPERSEDED; break;
+        case  5: crl->item.reason |= KSBA_CRLREASON_CESSATION_OF_OPERATION;
+          break;
+        case  6: crl->item.reason |= KSBA_CRLREASON_CERTIFICATE_HOLD; break;
+        case  8: crl->item.reason |= KSBA_CRLREASON_REMOVE_FROM_CRL; break;
+        case  9: crl->item.reason |= KSBA_CRLREASON_PRIVILEGE_WITHDRAWN; break;
+        case 10: crl->item.reason |= KSBA_CRLREASON_AA_COMPROMISE; break;
+        default: crl->item.reason |= KSBA_CRLREASON_OTHER; break;
+        }
+    }
+  else if (critical)
+    err = gpg_error (GPG_ERR_UNKNOWN_CRIT_EXTN);
+  
+  return err;
+}
+
 /* Parse the revokedCertificates SEQUENCE of SEQUENCE using a custom
    parser for efficiency and return after each entry */
 static gpg_error_t
@@ -1114,6 +1215,7 @@ parse_crl_entry (ksba_crl_t crl, int *got_entry)
   memcpy (crl->item.serial+numbuflen, tmpbuf+ti.nhdr, ti.length);
   crl->item.serial[numbuflen + ti.length] = ')';
   crl->item.serial[numbuflen + ti.length + 1] = 0;
+  crl->item.reason = 0;
 
   /* get the revocation time */
   err = _ksba_ber_read_tl (crl->reader, &ti);
@@ -1141,7 +1243,7 @@ parse_crl_entry (ksba_crl_t crl, int *got_entry)
   HASH (tmpbuf, ti.nhdr+ti.length);
   
   _ksba_asntime_to_iso (tmpbuf+ti.nhdr, ti.length,
-                        crl->item.revocation_date);
+                        ti.tag == TYPE_UTC_TIME, crl->item.revocation_date);
 
   /* if there is still space we must parse the optional entryExtensions */
   if (ndef)
@@ -1188,7 +1290,9 @@ parse_crl_entry (ksba_crl_t crl, int *got_entry)
           if (err)
             return err;
           HASH (tmpbuf, ti.nhdr+ti.length);
-          /* fixme: handle extension */
+          err = store_one_entry_extension (crl, tmpbuf, ti.nhdr+ti.length);
+          if (err)
+            return err;
         }
     }
 
